unit Exec;


{
Delphi-Source.de
Um ein anderes Programm zu starten sollte man CreateProcess benutzen,
da diese Funktion sehr viele Parameter kennt. Hier eine Funktion, die
den Umgang damit erleichtert. In AFilename wird der Dateiname des
Programms bergeben, in AParameter kann die Parameterzeile angegeben
werden, in ACurrentDir bergibt man ein alternatives Startverzeichnis,
bei einem leeren String wird das Verzeichnis der Exe benutzt. Setzt man
ACurrentDir auf ein nicht vorhandenes Verzeichnis, startet die Anwendung
nicht. Wenn AWait True ist, wartet das Programm darauf, dass das
aufgerufene Programm beendet wird.
bergibt man den optionalen Parameter AOnWaitProc, so wird diese Funktion
etwa alle 50 ms aufgerufen, bis das gestartete Programm wieder geschlossen
wird. ber die Parameter, die AOnWaitProc bergeben werden, kann man auf
den Prozess zugreifen und ihn ggf. beenden.
}



interface


USES Windows, SysUtils, Forms, MMsystem;


type
  TExecuteWaitEvent = procedure(const ProcessInfo: TProcessInformation;
                                const StartedAt: integer;
                                var ATerminate: Boolean) of object;

function ExecuteFile(const AFilename              : String;
                           AParameter, ACurrentDir: String;
                           AWait                  : Boolean;
                           AOnWaitProc            : TExecuteWaitEvent=nil): integer;


implementation



function ExecuteFile(const AFilename              : String;
                            AParameter, ACurrentDir: String;
                            AWait                  : Boolean;
                            AOnWaitProc            : TExecuteWaitEvent=nil): integer;
var
  si: TStartupInfo;
  pi: TProcessInformation;
  bTerminate: Boolean;
  StartedAt: integer;
begin
  bTerminate := False;

  if Length(ACurrentDir) = 0 then
    ACurrentDir := ExtractFilePath(AFilename);

  if ACurrentDir <> '' then
    if AnsiLastChar(ACurrentDir) = '\' then
      Delete(ACurrentDir, Length(ACurrentDir), 1);

  FillChar(si, SizeOf(si), 0);
  with si do begin
    cb := SizeOf(si);
    dwFlags := STARTF_USESHOWWINDOW;
    wShowWindow := SW_MINIMIZE;	//SW_NORMAL;
  end;

  FillChar(pi, SizeOf(pi), 0);
  AParameter := Format('"%s" %s', [AFilename, TrimRight(AParameter)]);

  StartedAt := TimeGetTime;
  if CreateProcess(Nil, PChar(AParameter), Nil, Nil, False,
                   CREATE_DEFAULT_ERROR_MODE or CREATE_NEW_CONSOLE or
                   NORMAL_PRIORITY_CLASS, Nil, PChar(ACurrentDir), si, pi)
    then
      try
        Result := 0;
        if AWait then
          while WaitForSingleObject(pi.hProcess, 50) <> Wait_Object_0 do
            begin
              if Assigned(AOnWaitProc) then
                begin
                  AOnWaitProc(pi, StartedAt, bTerminate);
                  if bTerminate then
                    begin
                      TerminateProcess(pi.hProcess, Cardinal(-1));
                      Result := -1;
                    end;
                end;

              Application.ProcessMessages;
            end;
      finally
        CloseHandle(pi.hProcess);
        CloseHandle(pi.hThread);
      end
    else Result := GetLastError;
end;




end.
