unit FViewer;


{*****************************************************************}
{                                                                 }
{  FViewer                                                        }
{                                                                 }
{  Source code viewer for UnitUsage                               }
{                                                                 }
{  Displays a RTF file containing a prepared pascal source file.  }
{  Adds a column with line numbers.                               }
{                                                                 }
{  Known bugs: Line numbers are sometimes not updated             }
{                                                                 }
{  (c) 2007 M. Braun                                              }
{  Contact: info@grizzlymotion.com                                }
{                                                                 }
{*****************************************************************}



interface

{$WARN SYMBOL_DEPRECATED OFF} {Use of MakeObjectInstance and FreeObjectInstance}


uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, RichEdit, Buttons, StrUtils, DelDirective;

type
  TForm_Viewer = class(TForm)
    StatusBar1: TStatusBar;
    Panel1: TPanel;
    RichEdit1: TRichEdit;
    Panel2: TPanel;
    SaveDialog1: TSaveDialog;
    Edit_SearchText: TEdit;
    SpeedButton_SaveAsRTF: TSpeedButton;
    SpeedButton_SearchText: TSpeedButton;
    Panel3: TPanel;
    Panel_Dummy: TPanel;
    PaintBox1: TPaintBox;
    procedure FormActivate(Sender: TObject);
    procedure SpeedButton_SaveAsRTFClick(Sender: TObject);
    procedure Edit_SearchTextChange(Sender: TObject);
    procedure SpeedButton_SearchTextClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure PaintBox1Paint(Sender: TObject);
    procedure RichEdit1SelectionChange(Sender: TObject);
    procedure RichEdit1MouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
  private
    { Private declarations }
    fOrigName,
    fTempFileName : string;
    PRichEdWndProc, POldWndProc: Pointer;
    procedure RichEdWndProc(var Msg: TMessage);
  public
    { Public declarations }
    PROCEDURE LoadFromTempFile(OrigName, TempFileName: string);
  end;

var
  Form_Viewer: TForm_Viewer;

implementation

{$R *.dfm}


PROCEDURE TForm_Viewer.LoadFromTempFile(OrigName, TempFileName: string);
BEGIN
  fTempFileName := TempFileName;
  fOrigName     := OrigName;
ENd;


procedure TForm_Viewer.FormActivate(Sender: TObject);
var
  f : file;
  i : integer;
  {
  SL: TStringList;
  ST: TStringStream;
  IsItalic: boolean;
  }
begin
  IF not(Fileexists(fTempFileName)) then
    exit;
  SpeedButton_SaveAsRTF.Enabled := false;
  Screen.Cursor := crHourGlass;
  Caption := 'UnitUsage-Viewer [' + fOrigName + ']';
  FileMode := 0;
  assignfile(f, fTempFileName);
  reset(f, 1);
  i := Filesize(f);
  closefile(f);
  RichEdit1.Lines.Clear;
  RichEdit1.PlainText := false;
  StatusBar1.Panels[0].Text := 'TRichEdit allocating memory...';
  Application.ProcessMessages;
  RichEdit1.MaxLength := i;
  StatusBar1.Panels[0].Text := 'TRichEdit reading file...';
  Application.ProcessMessages;
  RichEdit1.Lines.LoadFromFile(fTempFileName);
  StatusBar1.Panels[0].Text := '';
  Screen.Cursor := crDefault;
  SpeedButton_SaveAsRTF.Enabled := true;
  PaintBox1.invalidate;
  {
  ST := TStringStream.Create(SL.Text);
  RichEdit2.MaxLength := Length(SL.Text);
  RichEdit2.Lines.LoadFromStream(ST);
  ST.Free;
  }
end;


procedure TForm_Viewer.SpeedButton_SaveAsRTFClick(Sender: TObject);
begin
  SaveDialog1.FileName := ChangeFileExt(fOrigName, '.RTF');
  IF SaveDialog1.Execute then
    BEGIN
      IF CopyFile(PChar(fTempFileName), PChar(SaveDialog1.filename), false)
        then MessageDlg('File ' + SaveDialog1.filename + ' exported.', mtInformation, [mbOK], 0)
        else MessageDlg('Unable to write file ' + SaveDialog1.filename + '.', mtError, [mbOK], 0)
    END;
end;


procedure TForm_Viewer.Edit_SearchTextChange(Sender: TObject);
begin
  SpeedButton_SearchText.Enabled := Length(Edit_SearchText.Text) > 0;
end;


procedure TForm_Viewer.SpeedButton_SearchTextClick(Sender: TObject);
var
  i : integer;
begin
  i := RichEdit1.FindText(Edit_SearchText.Text, RichEdit1.SelStart+1, MaxLongint, []);
  IF i > 0 then
    begin
      // http://www.delphipraxis.net/topic49509_richedit+in+zeile+springen.html&highlight=richedit+cursor
      RichEdit1.SetFocus;
      RichEdit1.SelStart := i;
      RichEdit1.SelLength := Length(Edit_SearchText.Text);
      RichEdit1.Perform(EM_SCROLLCARET, 0, 0);
      RichEdit1.Perform(EM_SCROLL, SB_LINEDOWN, 0);
      PaintBox1.Refresh;
    end;
end;


procedure TForm_Viewer.FormCreate(Sender: TObject);
begin
  PRichEdWndProc := MakeObjectInstance(RichEdWndProc);
  POldWndProc    := Pointer(SetWindowLong(RichEdit1.Handle, GWL_WNDPROC, Integer(PRichEdWndProc)));
end;

procedure TForm_Viewer.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if Assigned(PRichEdWndProc) then
    begin
      SetWindowLong(RichEdit1.Handle, GWL_WNDPROC, Integer(POldWndProc));
      FreeObjectInstance(PRichEdWndProc);
    end;
end;


procedure TForm_Viewer.RichEdWndProc(var Msg: TMessage);
//http://mc-computing.com/Languages/RichEdit.htm
begin
  Msg.Result := CallWindowProc(POldWndProc, RichEdit1.Handle, Msg.Msg,
    Msg.wParam, Msg.lParam);

  if ((Msg.Msg = WM_VSCROLL) and (LOWORD(Msg.wParam) = SB_THUMBTRACK)) or
      (Msg.Msg = WM_MOUSEWHEEL)
    then PaintBox1.invalidate;
end;


procedure TForm_Viewer.PaintBox1Paint(Sender: TObject);
// Nachdem die Win32 Hilfe mich in die Irre geleitet hatte (Beschreibung von EM_CharFromPos verkehrt),
// half mir dies hier weiter:
// http://www.swissdelphicenter.ch/de/showcode.php?id=1213
var
  Y, L : integer;
  R, L0: integer;
  pt   : TPoint;
  Index: integer;
  Y1,Y2: integer;
  st   : string;
begin
  L  := -1000;
  Y1 := 0;
  Y2 := 0;
  L0 := 0;
  IF RichEdit1.Visible then
    begin
      PaintBox1.Canvas.Font.Name := 'Courier New';
      PaintBox1.Canvas.Font.Size := 8;
      PaintBox1.Canvas.Brush.Color := $00E0E0E0;
      PaintBox1.Canvas.Rectangle(-1, -1, PaintBox1.Width+1, PaintBox1.Height+1);
      Index := 0;
      FOR Y := 0 TO RichEdit1.ClientHeight-1 DO
        BEGIN
          pt:= point(1,y);
          R := RichEdit1.Perform(EM_CHARFROMPOS, 0, longint(@pt));
          IF R >= 0 then
            begin
              R := RichEdit1.Perform(EM_EXLINEFROMCHAR, 0, R);
              IF R <> L THEN
                BEGIN
                  L := R;
                  IF Index > 0 then
                    begin
                      st := IntToStr(L+1);
                      WITH PaintBox1.Canvas DO
                        TextOut(PaintBox1.Width-TextWidth(st+'W'), Y, st);
                    end;
                  IF Index = 0 THEN L0 := L;
                  IF Index = 1 THEN Y1 := Y;
                  IF Index = 2 THEN Y2 := Y;
                  Inc(Index);
                END;
            end;
        END;
      st := IntToStr(L0+1);
      WITH PaintBox1.Canvas DO
        TextOut(PaintBox1.Width-TextWidth(st+'W'), Y1+Y1-Y2, st);
    end;
  //RichEdit1.Perform(EM_GetScrollPos)
end;


procedure TForm_Viewer.RichEdit1SelectionChange(Sender: TObject);
VAR
  X, Y : integer;
begin
  PaintBox1.Invalidate;
  Y := RichEdit1.Perform(EM_EXLINEFROMCHAR, 0, RichEdit1.SelStart);
  // Didn't find a better way to determine X position  :(
  IF Y = 0
    then X := RichEdit1.SelStart
    else BEGIN
      X := 1;
      WHILE RichEdit1.Perform(EM_EXLINEFROMCHAR, 0, RichEdit1.SelStart-X) = Y
        DO Inc(X);
    END;
  StatusBar1.Panels[0].Text := IntToStr(Y+1)+' : ' + IntToStr(X);
end;


procedure TForm_Viewer.RichEdit1MouseWheel(Sender: TObject;
  Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint;
  var Handled: Boolean);
begin
  PaintBox1.invalidate;
end;

end.
